package com.onsiteservice.constant.constant;

/**
 * @author 潘维吉
 * @date 2020/3/3 10:10
 * @email 406798106@qq.com
 * @description 字典表常量
 * 0 1 表示布尔类型状态 可不定义内部类  定义字典紧贴主编码下方  除布尔值 其他编码从值 1 开始
 * mybatis的xml中的引用语法规则：${@package$subClass@Attr.getMethod}
 */
public class DictConstants {

    /**
     * ------------------系统字典数据------------------
     */

    /**
     * 字典类型 1.系统默认 2.自定义
     */
    public final static String SYS_DICT_CATEGORY = "SYS0000";

    /**
     * 启用状态 0 禁用 1 启用
     */
    public final static String AVAILABLE_STATUS = "SYS0001";

    /**
     * 是否状态 0 否 1 是
     */
    public final static String WHETHER_STATUS = "SYS0002";

    public class WhetherStatus {
        public static final int NO = 0; // 否
        public static final int YES = 1; // 是
    }

    /**
     * 是否删除 0 未删除 1 已删除
     */
    public final static String DELETE_STATUS = "SYS0003";

    public class DeleteStatus {
        public static final int NOT_DELETED = 0; // 未删除
        public static final int DELETED = 1; // 已删除
    }

    /**
     * 性别
     * mybatis xml中使用常量${@com.monorepo.constant.DictConstants$SEX@MAN}
     */
    public final static String SEX = "SYS0004";

    public class Sex {
        public static final int UNKNOWN = 0; // 未知
        public static final int MAN = 1; // 男
        public static final int WOMAN = 2; //女
    }

    /**
     * 系统参数 1 整型 2 字符串  3 富文本 4 图片 5 文件
     */
    public final static String SYS_PARAM_TYPE = "SYS0005";

    /**
     * 成功状态 0 失败 1 成功
     */
    public final static String SUCCESS_STATUS = "SYS0006";

    public class SuccessStatus {
        public static final int FAIL = 0; // 失败
        public static final int SUCCESS = 1; // 成功
    }

    /**
     * 管理平台
     */
    public final static String MANAGE_PLATFORM = "SYS0007";

    public class ManagePlatform {
        public static final int OPERATION = 1; // 运营平台
        public static final int ORGANIZATION = 2; // 公司平台
    }

    /**
     * ------------------业务自定义字典数据------------------
     */

    /**
     * 平台类型
     */
    public final static String PLATFORM = "BIZ0001";

    /**
     * mybatis xml中使用常量 ${@com.monorepo.constant.DictConstants$Platform@USER_APP}
     */
    public class Platform {
        public static final int USER_APP = 1; // 用户端小程序
        public static final int MANAGE_APP = 2; // 管理端小程序
    }


    /**
     * 推送Job发送状态
     */
    public final static String PUSH_JOB_STATUS = "BIZ0002";

    public class PushJobStatus {
        public static final int PENDING = 1;   // 待发送
        public static final int SENDING = 2;   // 发送中
        public static final int FINISH = 3;    // 发送完成
    }


    /**
     * 订单状态 10待付款 20待派单 30待接单 40服务中 50已结束"
     */
    public final static String ORDER_STATUS = "BIZ0009";

    public class OrderStatus {
        public static final int WAIT_PAY = 10;   // 待支付
        public static final int PAYING = 11;   // 支付中
        public static final int WAIT_APPOINT = 20;   // 待派单
        public static final int WAIT_ACCEPT = 30;    // 待接单
        public static final int SERVICING = 40;    // 服务中
        public static final int FINISH = 50;    // 已结束
    }

    /**
     * 数据来源 1小程序 2日照通
     */
    public final static String ADD_SOURCE = "BIZ0004";

    public class AddSource {
        public static final int WX = 1; // 微信小程序
        public static final int RZT = 2; // 日照通
    }

    /**
     * 取消订单 0未取消 1支付超时取消 2用户发起手动取消 3手动取消已确认 4取消支付取消 5管理员拒单
     */
    public final static String ORDER_CANCEL_STATUS = "";

    public class OrderCancelStatus {
        public static final int NO = 0; // 0未取消
        public static final int UNPAY = 1; // 1支付超时取消
        public static final int MANUAL = 2; // 2用户发起手动取消
        public static final int CONFIRM = 3; // 3手动取消已确认
        public static final int CANCEL_PAY = 4; // 4取消支付取消
        public static final int REFUSE = 5; // 5管理员拒单
    }

    /**
     * 退款状态 1发起 2退款中 3退款成功
     */
    public final static String ORDER_REFUND_STATUS = "BIZ0005";

    public class OrderRefundStatus {
        public static final int NONE = 0; // 未发起
        public static final int BEGIN_REFUND = 1; // 发起
        public static final int REFUNDING = 2; // 退款中
        public static final int FINISH = 3; // 退款成功
    }

    /**
     * 资讯状态：1新建 2发布 3撤销
     */
    public final static String NEWS_STATUS = "BIZ0007";

    public class NewsStatus {
        public static final int NEW = 1; // 新建
        public static final int RELEASE = 2; // 发布
        public static final int CANCEL = 3; // 撤销
    }

    /**
     * 评价状态 0未评价 1未审核 2已审核
     */
    public final static String COMMENT_STATUS = "";

    public class CommentStatus {
        public static final int NO = 0; // 0未评价
        public static final int UN_CONFIRM = 1; // 1未审核
        public static final int CONFIRM = 2; // 2已审核
    }

    /**
     * 订单环节状态 1未处理 2已处理 3已拒绝 4已取消
     */
    public final static String ORDER_NODE_STATUS = "";

    public class OrderNodeStatus {
        public static final int UNDO = 1; // 1未处理
        public static final int DONE = 2; // 2已处理
        public static final int REFUSE = 3; // 3已拒绝
        public static final int CANCEL = 4; // 4已取消
    }

    /**
     * 客户(被服务人自理)状态 1:客户完全不能自理 2:客户半自理
     */
    public final static String ClientStatus = "BIZ0003";

    public class ClientStatus {
        public static final int CANT_SELF_CARE = 1; // 完全不能自理
        public static final int SEMI_SELF_CARE = 2; // 半自理
    }

    /**
     * 护工等级
     */
    public final static String CAREGIVER_LEVEL = "BIZ0006";

    public class CaregiverLevel {
        public static final int ONE = 1; // 初级护工
        public static final int TWO = 2; // 中级护工
        public static final int THREE = 3; // 高级护工
        public static final int FOUR = 4; // 金牌护工
    }

    /**
     * 护工提现审批状态
     */
    public final static String CAREGIVER_ACCOUNT_APPROVE_STATUS = "BIZ0010";

    public class CaregiverAccountApproveStatus {
        public static final int UNDO = 1; // 未审批
        public static final int PASS = 2; // 已通过
        public static final int REFUSE = 3; // 未通过
    }

    /**
     * 护工钱包流水类型
     */
    public final static String CAREGIVER_ACCOUNT_FLOW_TYPE = "BIZ0011";

    public class CaregiverAccountFlowType {
        public static final int COMMISSION = 1; // 结佣
        public static final int WITHDRAWAL = 2; // 提现
    }

    /**
     * 账单表类型
     */
    public final static String SERVICES_BILL_TYPE = "BIZ0012";

    public class ServicesBillType {
        public static final int INCOME = 1; // 收入
        public static final int EXPEND = 2; // 支出
    }

}
